//Copyright (C) 2004, Brian Enigma <enigma at netninja.com>
//This file is part of MagicCodes.
//
//MagicCodes is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//MagicCodes is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with Foobar; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package org.ninjasoft.magiccodes.plugins;

/**
 * Vigenere Decryption
 * @author enigma
 */
public class VigenereDecrypt implements Plugin {
	public String getName() {
		return "Vigenere Decryption";
	}
	public String getDescription() {
		return "Decrypt Vigenere ciphers";
	}
	public boolean usesKey() {
		return true;
	}
	public boolean isInformational() {
		return false;
	}
	public int[] doAction(int[] in, int[] key) {
        int[] result = new int[in.length];
        String keyStr = strip(key);
        for (int i=0; i<in.length; i++) {
            char letter = (char) in[i];
            if (((letter >= 'a') && (letter <= 'z')) ||
                ((letter >= 'A') && (letter <= 'Z'))) {
                    letter = decodeLetters(letter, keyStr.charAt(i % keyStr.length()));
            }
            result[i] = letter;
        }    
        return result;
	}
    public char decodeLetters(char letter1, char letter2) {
        int l1 = Character.toUpperCase(letter1) - 'A';
        int l2 = Character.toUpperCase(letter2) - 'A';
        int l3 = (l1 - l2 + 26) % 26;
        return (char) (l3 + 'A');
    }
    public String strip(int[] s) {
        StringBuffer result = new StringBuffer();
        for (int i=0; i<s.length; i++) {
            char c = Character.toUpperCase((char) s[i]);
            if ((c >= 'A') && (c <= 'Z'))
                result.append(c);
        }
        return result.toString();
    }
}
