//Copyright (C) 2004, Brian Enigma <enigma at netninja.com>
//This file is part of MagicCodes.
//
//MagicCodes is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//MagicCodes is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with Foobar; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package org.ninjasoft.magiccodes.plugins;

import java.text.DecimalFormat;

/**
 * Compute the index of coincidence-related data for XOR ciphers, using
 * the pseudocode:
 * for i is 1 to length(cyphertext) do
 *     cyphertext2 = shift(cyphertext,i)
 *     for j is 1 to length(cyphertext) do
 *           if (cyphertext[i] = cyphertext2[j]) then do 
 *                counter = counter +1
 *     index = (counter / length(cyphertext))
 *     if index > .4 then do return i
 *
 * (Note: you might have to change .4 to something else depending on your
 * character set.)
 * as described on http://md2600.net/newsletter/issue2/122.html
 * 
 * @author enigma
 */
public class XorAnalysis implements Plugin {
    private DecimalFormat format = new DecimalFormat("###");
    private DecimalFormat dotFormat = new DecimalFormat("0.0000");
	public String getName() {
		return "XOR:Analysis";
	}
	public String getDescription() {
		return "Calculates the Index of Coincidence for XOR ciphers";
	}
	public boolean usesKey() {
		return false;
	}
	public boolean isInformational() {
		return true;
	}
	public int[] doAction(int[] in, int[] key) {
        // String buffer for output
        StringBuffer result = new StringBuffer();
        result.append("XOR Index of Coincidence Data\n");
        for (int shift=1; shift<in.length; shift++) {
            int counter = 0;
            //cyphertext2 = shift(cyphertext,i)
            for (int i=0; i<in.length; i++) {
                  if (in[i] == in[(i + shift) % in.length]) 
                       counter++;
            }
            double index = (double) counter / (double) in.length;
            result.append("Shift:");
            result.append(format.format(shift));
            result.append("  Index:");
            result.append(dotFormat.format(index));
            result.append("\n");
            //if index > .4 then do return i
        }
        // Flatten to an array for returning
        //System.out.println(result.toString());
        int[] resultArray = new int[result.length()];
        for (int i=0; i<resultArray.length; i++)
            resultArray[i] = result.charAt(i);
        return resultArray;
	}
}
